@@ -7,7 +7,7 @@ require Exporter;
 @Set::Tiny::ISA = qw(Exporter);
 @Set::Tiny::EXPORT_OK = qw(set);
 
-$Set::Tiny::VERSION = '0.02';
+$Set::Tiny::VERSION = '0.03';
 
 sub new {
     my $class = shift;
@@ -94,7 +94,16 @@ sub union {
     return $class->new( keys %{$_[0]}, keys %{$_[1]} );
 }
 
-sub intersection { $_[0]->difference( $_[0]->clone->difference($_[1]) ) }
+sub intersection {
+    my $class = ref $_[0];
+    return $class->new( grep { exists($_[0]->{$_}) } keys %{$_[1]} );
+}
+
+sub intersection2 {
+    my $class = ref $_[0];
+    my ($a, $b) = $_[0]->size > $_[1]->size ? ($_[0], $_[1]) : ($_[1], $_[0]);
+    return $class->new( grep { exists($a->{$_}) } keys %{$b} );
+}
 
 sub symmetric_difference { $_[0]->clone->invert(keys %{$_[1]}) }
 
@@ -118,7 +127,7 @@ Set::Tiny - Simple sets of strings
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =head1 SYNOPSIS
 
@@ -267,6 +276,13 @@ Returns a new set containing both the elements of this set and I<set>.
 Returns a new set containing the elements that are present in both this
 set and I<set>.
 
+=head2 intersection2( I<set> )
+
+Like C<intersection()>, but orders the sets by size before comparing their
+elements. This results in a small overhead for small, evenly sized sets, but
+a large speedup when comparing bigger (~ 100 elements) and very unevenly
+sized sets.
+
 =head2 difference( I<set> )
 
 Returns a new set containing the elements of this set with the elements
